package choco;

import static org.chocosolver.solver.search.strategy.Search.minDomLBSearch;

import java.io.IOException;

import org.chocosolver.solver.Model;
import org.chocosolver.solver.Solver;
import org.chocosolver.solver.constraints.nary.circuit.CircuitConf;
import org.chocosolver.solver.variables.IntVar;

public class tsp_next_aula {

	public static void main(String[] args) throws IOException {
				
		int maxd = 600; 
		
		//model.element(cost[i], dist[i], next[i],0).post();  // cost[i] = dist[next[i]]
		
	    int [][] dist = { // the adjacency graph of the problem
	    	{   0, 107, 241, 190, 124,  80, 316,  76, 152, 157, 283, 133, 113, 297, 228},
	    	{ 107,   0, 148, 137,  88, 127, 336, 183, 134,  95, 254, 180, 101, 234, 175},
	    	{ 241, 148,   0, 374, 171, 259, 509, 317, 217, 232, 491, 312, 280, 391, 412},
	    	{ 190, 137, 374,   0, 202, 234, 222, 192, 248,  42, 117, 287,  79, 107,  38},
	    	{ 124,  88, 171, 202,   0,  61, 392, 202,  46, 160, 319, 112, 163, 322, 240},
	    	{  80, 127, 259, 234,  61,   0, 386, 141,  72, 167, 351,  55, 157, 331, 272},
	    	{ 316, 336, 509, 222, 392, 386,   0, 233, 438, 254, 202, 439, 235, 254, 210},
	    	{  76, 183, 317, 192, 202, 141, 233,   0, 213, 188, 272, 193, 131, 302, 233},
	    	{ 152, 134, 217, 248,  46,  72, 438, 213,   0, 206, 365,  89, 209, 368, 286},
	    	{ 157,  95, 232,  42, 160, 167, 254, 188, 206,   0, 159, 22,   57, 149,  80},
	    	{ 283, 254, 491, 117, 319, 351, 202, 272, 365, 159,   0, 404, 176, 106,  79},
	    	{ 133, 180, 312, 287, 112,  55, 439, 193,  89, 220, 404,   0, 210, 384, 325},
	    	{ 113, 101, 280,  79, 163, 157, 235, 131, 209,  57, 176, 210,   0, 186, 117},
	    	{ 297, 234, 391, 107, 322, 331, 254, 302, 368, 149, 106, 384, 186,   0,  69},
	    	{ 228, 175, 412,  38, 240, 272, 210, 233, 286,  80,  79, 325, 117,  69,   0}
	    };
	 	int n = 15;
		
		Model model = new Model(" TSP ");
		Solver s = model.getSolver();
		
		//int backLimit = 1000; s.limitBacktrack(1000);
		//int solLimit = 15; s.limitSolution(solLimit);
		//int timeLimit = 120000; s.limitTime(timeLimit);
		
		IntVar [] next = model.intVarArray("N", n, 0, n-1);
		
		CircuitConf conf = CircuitConf.LIGHT; // ALL, FIRST, LIGHT, RD
		model.circuit(next, 0, conf).post();
		
		IntVar [] cost = model.intVarArray("C", n, 0, maxd);
		IntVar totCost = model.intVar("X", 0, n*maxd);	
		
		model.sum(cost, "=", totCost).post();
		
		for(int i = 0; i < n; i++) 
			model.element(cost[i], dist[i], next[i],0).post();  // cost[i] = dist[next[i]]
		
		model.setObjective(Model.MINIMIZE, totCost);	
		s.setSearch(minDomLBSearch(next));

		while (s.solve()){
			System.out.print("\n -- solution " + s.getSolutionCount() + "\t= " + totCost.getValue());
			System.out.println(" found after " + s.getTimeCount() + " secs");
			for(int i = 0; i< n; i++) System.out.print(" " + next[i].getValue()); System.out.println();
		}
		System.out.print( "\n !!! No (more) solutions");
		System.out.println(" after " + s.getTimeCount() + " secs");
		System.out.println('\n');
	}
}
